// $rgbimutil.cpp 1.5 milbo$ RGB image utilities

#include "all.hpp"

//-----------------------------------------------------------------------------
void CropRgbImage (RgbImage &Img,                                                // io
                   int nTopCrop, int nBottomCrop, int nLeftCrop, int nRightCrop, // in
                   bool fWidthDivisibleBy4)                                      // in
{
int width = Img.width;
int height = Img.height;
if (fWidthDivisibleBy4)
    nLeftCrop += (Img.width - nRightCrop - nLeftCrop) % 4; // ensure width is div by 4
int nNewWidth = Img.width - nLeftCrop - nRightCrop;
int nNewHeight = Img.height - nTopCrop - nBottomCrop;
if (nTopCrop < 0 || nBottomCrop < 0 || nLeftCrop < 0 || nRightCrop < 0)
    Err("you can't specify a crop less than 0");
if (nNewWidth <= 0)
    Err("specified left or right crop would cause a width less than or equal to zero");
if (nNewWidth > width)
    Err("specified left or right crop would cause a width bigger than current width");
if (nNewHeight <= 0)
    Err("specified top or bottom crop would cause a height less than or equal to zero");
if (nNewHeight > height)
    Err("specified top or bottom crop would cause a height bigger than current height");

RgbImage OutImg(nNewWidth, nNewHeight);

for (int iy = 0; iy < nNewHeight; iy++)
    for (int ix = 0; ix < nNewWidth; ix++)
        OutImg(ix, iy) = Img(ix + nLeftCrop, iy + nBottomCrop);

Img = OutImg;
}

//-----------------------------------------------------------------------------
void ConvertRgbImageToGray (Image &OutImg,          // out
                            const RgbImage &Img)    // in
{
int width = Img.width;
int height = Img.height;

OutImg.dim(width, height);

for (int iy = 0; iy < height; iy++)
    for (int ix = 0; ix < width; ix++)
        OutImg(ix, iy) = (byte)RgbToGray(Img(ix + ((height - 1 - iy) * width)));
}
