// $matvec.cpp 1.5 milbo$ routines for handling vectors of matrices
// This originally used C arrays and was changed to use STL vectors.

#include "all.hpp"

//-----------------------------------------------------------------------------
// Read a matrix from a file, put it into pMatV.
//
// If pFile is NULL then it is opened.  If pFile is already open then we read
// from it (and sFile isn't used except for user messages).
//
// Matrix vector file format:
//
// # Optional comments prefixed by #
// Matrices one after each other, in standard format (see Mat::write function header)
//
// Actually comments can appear anywhere except within the { and } of a matrix.
// The starting character # must always be in the FIRST COLUMN.

void ReadMatVec (MatVec *pMatV,         // out
    const char *sFile, FILE *pFile)     // in
{
bool fCloseFile = false;
if (pFile == NULL)
    {
    ASSERT(sFile);
    pFile = Fopen(sFile, "r") ;
    fCloseFile = true;
    }
if (!sFile) // make sure we have something to print in user messages
    sFile = "file";

pMatV->clear();     // erase what's currently in MatV
Mat TempMat;
TempMat.read(sFile, pFile, NULL);
pMatV->push_back(TempMat);

if (fCloseFile)
    fclose(pFile);
}
