# $stasm\ms\makefile.ms 1.4 milbo$ makefile for stasm
# This was created for Microsoft Visual C++ version 6.0 under WinXP.
#
# Release build: mkdir Release; nmake -nologo CFG=Release -f makefile.ms
# Debug build:   mkdir Debug;   nmake -nologo CFG=Debug   -f makefile.ms
#
# milbo durban aug07

!IF "$(CFG)" != "Release" && "$(CFG)" != "Debug" && "$(CFG)" != "Profile"
!ERROR Invalid configuration "$(CFG)" specified.
!ENDIF

BASE_DIR=  ..
GSL_DIR=   $(BASE_DIR)\gsl
IMAGE_DIR= $(BASE_DIR)\image
JPEG_DIR=  $(BASE_DIR)\jpeg
MAT_DIR=   $(BASE_DIR)\mat
ROWLEY_DIR=$(BASE_DIR)\rowley
STASM_DIR= $(BASE_DIR)\stasm
DATA_DIR=  $(BASE_DIR)/stasm/data
CV_DIR=    $(BASE_DIR)/opencv

INCL=-I$(GSL_DIR) -I$(IMAGE_DIR) -I$(JPEG_DIR) -I$(MAT_DIR) \
     -I$(ROWLEY_DIR) -I$(STASM_DIR) \
     -I$(CV_DIR)/cxcore/include -I$(CV_DIR)/cv/include \
     -I$(CV_DIR)/otherlibs/highgui

!IF  "$(CFG)" == "Debug"
# -Zi is for a debugging build
# -W3 is warning level 3
# -MLd is for the single threaded static debugging runtime library
# -GX is for synchronous exception handling to keep the library stream header files happy
# -DGSL_RANGE_CHECK tells the mat lib do do range checking
# No need to define _DEBUG, compiler does it for us if -MLd flag is on
OUTDIR=.\Debug
CFLAGS=-nologo -Zi -W3 -MLd -GX -DGSL_RANGE_CHECK $(INCL) -Fo"$(OUTDIR)/" -c
LFLAGS=-nologo -debug:full -nodefaultlib:LIBC
LIBS=$(GSL_DIR)\lib_ms\mini_gslMLd.lib $(GSL_DIR)\lib_ms\gslcblasMLd.lib \
     $(JPEG_DIR)\lib_ms\libjpeg.lib \
     $(CV_DIR)\lib_ms\cxcore.lib $(CV_DIR)\lib_ms\cv.lib \
     $(CV_DIR)\lib_ms\highgui.lib
!ENDIF

!IF  "$(CFG)" == "Release"
# -02 is max level of optimization
# -Oa means assume no pointer aliasing
# -W3 is warning level 3
# -GX is for synchronous exception handling to keep the library stream header files happy
# Use -Zi and -debug:full to do a release build with debugging symbols
OUTDIR=.\Release
CFLAGS=-nologo -O2 -Oa -W3 -GX $(INCL) -Fo"$(OUTDIR)/" -c
LFLAGS=-nologo
LIBS=$(GSL_DIR)\lib_ms\mini_gslML.lib $(GSL_DIR)\lib_ms\gslcblasML.lib \
     $(JPEG_DIR)\lib_ms\libjpeg.lib \
     $(CV_DIR)\lib_ms\cxcore.lib $(CV_DIR)\lib_ms\cv.lib $(CV_DIR)\lib_ms\highgui.lib
!ENDIF

!IF  "$(CFG)" == "Profile"
OUTDIR=.\Debug
CFLAGS=-nologo -O2 -W3 -GX $(INCL) -Fo"$(OUTDIR)/" -c
LFLAGS=-nologo -nodefaultlib:LIBC \
-map -mapinfo:exports -mapinfo:fixups -mapinfo:lines -fixed:no
LIBS=$(GSL_DIR)\lib_ms\mini_gslML.lib $(GSL_DIR)\lib_ms\gslcblasML.lib \
     $(JPEG_DIR)\lib_ms\libjpeg.lib \
     $(CV_DIR)\lib_ms\cxcore.lib $(CV_DIR)\lib_ms\cv.lib $(CV_DIR)\lib_ms\highgui.lib
!ENDIF

all: stasm.exe

clean:
	del stasm.* $(OUTDIR)\*.obj Release\*.obj Debug\*.obj
 	del log *.bmp *.pdb *.ilk *.opt *.cod default.tag 

test:
	stasm -v $(DATA_DIR)/testImage.jpg
	@echo The following diff and cmp should give no output:
	diff $(DATA_DIR)/test-vj-log log
	cmp $(DATA_DIR)/results-vj-testImage.bmp results-vj-testImage.bmp
	del log
	del results-vj-testImage.bmp
	stasm $(DATA_DIR)/testImage.jpg
	@echo The following diff and cmp should give no output:
	diff $(DATA_DIR)/test-rowley-log log
	cmp $(DATA_DIR)/results-rowley-testImage.bmp results-rowley-testImage.bmp
	del log
	del results-rowley-testImage.bmp

STASM_OBJ= \
	$(OUTDIR)\asm.obj  \
	$(OUTDIR)\asmfile.obj  \
	$(OUTDIR)\find.obj  \
	$(OUTDIR)\follow.obj  \
	$(OUTDIR)\forward.obj  \
	$(OUTDIR)\imfile.obj  \
	$(OUTDIR)\imshape.obj  \
	$(OUTDIR)\imutil.obj  \
	$(OUTDIR)\initnet.obj  \
	$(OUTDIR)\jpegutil.obj  \
	$(OUTDIR)\main.obj  \
	$(OUTDIR)\mat.obj  \
	$(OUTDIR)\matvec.obj  \
	$(OUTDIR)\prof.obj  \
	$(OUTDIR)\rgbimutil.obj  \
	$(OUTDIR)\rowley.obj  \
	$(OUTDIR)\search.obj  \
	$(OUTDIR)\startshape.obj  \
	$(OUTDIR)\tclHash.obj \
	$(OUTDIR)\sparsemat.obj  \
	$(OUTDIR)\util.obj \
	$(OUTDIR)\violajones.obj \
	$(OUTDIR)\wrbmp.obj

# we include setargv.obj so stasm does command line filename globbing

stasm.exe: $(STASM_OBJ) makefile.ms
	link $(LFLAGS) -subsystem:console -out:stasm.exe $(STASM_OBJ) \
"c:\Program Files\Microsoft Visual Studio\VC98\lib\setargv.obj" $(LIBS)

$(STASM_DIR)\all.hpp: \
	$(IMAGE_DIR)\image.hpp \
	$(IMAGE_DIR)\imfile.hpp \
	$(IMAGE_DIR)\imutil.hpp \
	$(IMAGE_DIR)\jpegutil.hpp \
	$(IMAGE_DIR)\rgbimutil.hpp \
	$(MAT_DIR)\mat.hpp \
	$(MAT_DIR)\matvec.hpp \
	$(MAT_DIR)\matview.hpp \
	$(ROWLEY_DIR)\find.hpp \
	$(ROWLEY_DIR)\follow.hpp \
	$(ROWLEY_DIR)\forward.hpp \
	$(ROWLEY_DIR)\initnet.hpp \
	$(ROWLEY_DIR)\list.hpp \
	$(ROWLEY_DIR)\search.hpp \
	$(STASM_DIR)\asm.hpp \
	$(STASM_DIR)\asmfile.hpp \
	$(STASM_DIR)\atface.hpp \
	$(STASM_DIR)\colors.hpp \
	$(STASM_DIR)\cvec.hpp \
	$(STASM_DIR)\imshape.hpp \
	$(STASM_DIR)\landmarks.hpp \
	$(STASM_DIR)\misc.hpp \
	$(STASM_DIR)\prof.hpp \
	$(STASM_DIR)\rowley.hpp \
	$(STASM_DIR)\startshape.hpp \
	$(STASM_DIR)\sparsemat.hpp \
	$(STASM_DIR)\util.hpp \
	$(STASM_DIR)\violajones.hpp

$(OUTDIR)\asm.obj: $(STASM_DIR)\asm.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\asm.cpp

$(OUTDIR)\asmfile.obj: $(STASM_DIR)\asmfile.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\asmfile.cpp

$(OUTDIR)\imshape.obj: $(STASM_DIR)\imshape.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\imshape.cpp

$(OUTDIR)\main.obj: $(STASM_DIR)\main.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\main.cpp

$(OUTDIR)\prof.obj: $(STASM_DIR)\prof.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\prof.cpp

$(OUTDIR)\rowley.obj: $(STASM_DIR)\rowley.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\rowley.cpp

$(OUTDIR)\startshape.obj: $(STASM_DIR)\startshape.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\startshape.cpp

$(OUTDIR)\sparsemat.obj: $(STASM_DIR)\sparsemat.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\sparsemat.cpp

$(OUTDIR)\util.obj: $(STASM_DIR)\util.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\util.cpp

$(OUTDIR)\violajones.obj: $(STASM_DIR)\violajones.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(STASM_DIR)\violajones.cpp

$(OUTDIR)\mat.obj: $(MAT_DIR)\mat.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(MAT_DIR)\mat.cpp

$(OUTDIR)\matview.obj: $(MAT_DIR)\matview.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(MAT_DIR)\matview.cpp

$(OUTDIR)\matvec.obj: $(MAT_DIR)\matvec.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(MAT_DIR)\matvec.cpp

$(OUTDIR)\imfile.obj: $(IMAGE_DIR)\imfile.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(IMAGE_DIR)\imfile.cpp

$(OUTDIR)\imutil.obj: $(IMAGE_DIR)\imutil.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(IMAGE_DIR)\imutil.cpp

$(OUTDIR)\rgbimutil.obj: $(IMAGE_DIR)\rgbimutil.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(IMAGE_DIR)\rgbimutil.cpp

$(OUTDIR)\jpegutil.obj: $(IMAGE_DIR)\jpegutil.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(IMAGE_DIR)\jpegutil.cpp

$(OUTDIR)\forward.obj: $(ROWLEY_DIR)\forward.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(ROWLEY_DIR)\forward.cpp

$(OUTDIR)\follow.obj: $(ROWLEY_DIR)\follow.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(ROWLEY_DIR)\follow.cpp

$(OUTDIR)\imu.obj: $(ROWLEY_DIR)\imu.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(ROWLEY_DIR)\imu.cpp

$(OUTDIR)\find.obj: $(ROWLEY_DIR)\find.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(ROWLEY_DIR)\find.cpp

$(OUTDIR)\initnet.obj: $(ROWLEY_DIR)\initnet.cpp $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(ROWLEY_DIR)\initnet.cpp

$(OUTDIR)\search.obj: $(ROWLEY_DIR)\search.cpp $(STASM_DIR)\all.hpp $(ROWLEY_DIR)\list.hpp
	cl $(CFLAGS) $(ROWLEY_DIR)\search.cpp

$(OUTDIR)\tclHash.obj: $(ROWLEY_DIR)\tclHash.c $(STASM_DIR)\all.hpp $(ROWLEY_DIR)\tclHash.h
	cl $(CFLAGS) $(ROWLEY_DIR)\tclHash.c

$(OUTDIR)\wrbmp.obj: $(JPEG_DIR)\wrbmp.c $(STASM_DIR)\all.hpp
	cl $(CFLAGS) $(JPEG_DIR)\wrbmp.c
