// $asm.hpp 1.5 milbo$ routines for active shape models

#if !defined(asm_hpp)
#define asm_hpp

static const int    MAX_NBR_LEVS         = 4;   // stasm code supports up to this many lev in image pyr
static const int    MAX_NBR_SUBPROFS     = 1;   // stasm code supports up this many subprofs per prof
static const int    CONF_nAsmFileVersion = 0;   // ASM file version (part of the ASM file magic nbr)
static const int    CONF_nEigs           = 20;  // nbr of eig values to use in model
static const double CONF_BMax            = 1.8; // limit b in shape model to this
static const int    CONF_nEigsLev0       = 0;   // 1D: nEigs to use at level 0, 0 to use CONF_nEigs
static const int    CONF_BMaxLev0        = 0;   // 1D: BMax to use at level 0
static const int    CONF_nEigsLev0_2d    = 26;  // as above but for 2D
static const double CONF_BMaxLev0_2d     = 2.0; // as above
static const int    CONF_nEigsFinal      = 22;  // nEigs for final iteration of search at each level
static const double CONF_BMaxFinal       = 3.0;
static const int    CONF_nPixSearch      = 3;   // how far along the whisker do we search, each side
static const int    CONF_nPixSearch2d    = 2;   // used if there are ANY 2D profs at pyr level 0
static const int    CONF_nMaxSearchIters = 4;
static const int    CONF_nQualifyingDisplacments    = 75;
static const int    CONF_nMaxSearchIters2d          = 4;
static const int    CONF_nQualifyingDisplacments2d  = 90;

typedef cvec<SparseMat> SparseMatVec;

typedef struct tAsmLev          // ASM data for one level: level 0 is full size
    {
    int             nEigs;      // nEigs for LimitB()
    int             nEigsFinal; // nEigs for for the final search iteration
    double          BMax;       // for LimitB()
    double          BMaxFinal;
    SHAPE           MeanShape;  // FileMeanShape, scaled for this level

    cvec<MatVec>    Covars;     // array[nPoints][nSubProfs] of INVERSE of
                                // profile covar matrices, each is mat nPoints x nPoints

    cvec<SparseMatVec> SparseCovars;   // same as Covars, but sparse arrays

    cvec<VecVec>    Profs;      // array[nPoints][nSubProfs] of landmark profile
                                // row vecs, each is 1 x nPoints

    cvec<unsigned>  ProfSpecs;  // see PROF_ defs in prof.hpp
    Vec             EigVals;    // n x 1 vector, EigVals scaled for this level
    }
tAsmLev;

typedef struct tAsmModel        // an ASM model
    {
    int    iModel;              // used if models are stacked in series
    int    nPoints;             // number of landmarks
    int    nStartLev;           // start the search at this level in image pyramid
    double PyrRatio;
    int    PyrReduceMethod;     // one of IM_NEAREST_PIXEL IM_BILINEAR IM_AVERAGE_ALL
    int    nMaxSearchIters;
    int    nQualifyingDisplacments;
    int    nTrim;               // covariance trim
    double SigmoidScale;        // specifies how curvy the profile equalization sigmoid is
    int    nScaledFaceWidth;    // if >0 then prescale input face to this size
    int    nMaxOffset;          // max dist from landmark when searching, in pixels
    bool   fExplicitPrevNext;   // use iPrev and iNext in LandTab? see landmarks.hpp
    double NormalizedProfLen;
    SHAPE  FileMeanShape;       // mean shape read from ASM file
    Mat    EigVecs;             // n x n matrix
    Mat    EigInverse;          // EigVecs inverted
    tAsmLev AsmLevs[MAX_NBR_LEVS]; // model for each pyramid level
    }
tAsmModel;

double GetPyrScale(int iLev, double PyrRatio);
double GetX(double X, int iOffset, int iOrthOffset, double DeltaX, double DeltaY);
double GetY(double Y, int iOffset, int iOrthOffset, double DeltaX, double DeltaY);
bool fPointUsed(const SHAPE &Shape, int iPoint);
void SetUnusedPointsToZero(SHAPE &Shape, const SHAPE &RefShape);
SHAPE TransformShape(const SHAPE &Shape, const Mat &TransformMat);
Mat AlignShape(SHAPE &Shape, const SHAPE &AnchorShape, const Vec *pWeights=NULL);

SHAPE AsmSearch(const RgbImage &RgbImg, // in: find face features in this image
    const bool fViolaJones,             // in: true to use VJ detector, else Rowley
    const char sImageFile[],            // in: file path for RgbImg, for err msgs
    const char sDataDir[]);             // in: directory for data files

extern double gNormalizedProfLen;

#endif // asm_hpp
