// $atface.hpp 1.5 milbo$ face attributes

// Face attribute bits, used in first part of string preceding each shape
// array (before file name) in shape file e.g. "0002 B0002_01.pgm" means
// that the face in image B0002_01.pgm is wearing glasses.
//
// The same defines are used to specify attributes of individual landmarks
// in landmarks.hpp:gLandTab e.g. FA_Glasses means that this landmark is
// obscured if the subject is wearing glasses
//
// FA_BadImage marking is subjective and not entirely consistent.  I just
// wanted to weed out images that seemed likely to confuse the classifier
// during training.  Certainly all _really_ bad images are marked FA_BadImage; 
// marking is inconsistent only on marginally bad images.
//
// FA_NnFailed means that the Rowley neural net detector did not find one
// face and two eyes in the image.  Typically this means that the face is
// of poor quality or is wearing glasses. ("Nn" stands for "neural net").

#if !defined(atface_hpp)
#define atface_hpp

#define FA_BadImage   0x01   // image is "bad" in some way (blurred, face tilted, etc.)
#define FA_Glasses    0x02   // face is wearing specs
#define FA_Beard      0x04   // beard including possible mustache
#define FA_Mustache   0x08   // mustache but no beard occluding chin or cheeks
#define FA_Obscured   0x10   // faces is obscured e.g. by subject's hand
#define FA_EyesClosed 0x20   // eyes closed (partially open is not considered closed)
#define FA_Expression 0x40   // non-neutral expression on face
#define FA_NnFailed   0x80   // Rowley search failed (does not return 1 face with 2 eyes)
#define FA_Synthesize 0x100  // for synthesis of landmark profile from twin landmark
#define FA_Remarked   0x800  // one or more landmarks manually re-marked
#define FA_ViolaJones 0x1000 // landmarks are from face located using Viola Jones detector
#define FA_Rowley     0x2000 // landmarks are from face located using Rowley face detector
#define FA_Test1      0x4000 // test set 1
#define FA_Test2      0x8000 // test set 2

#endif // atface_hpp
