// $cvec.hpp 1.5 milbo$ cvec is like STL vector but
// checks for out of range indices when _DEBUG is defined.
//
// Lifted from Ulrich Breymann "Designing Components with the C++ STL".
// If _DEBUG is not defined the MSC compiler generates code with the same
// efficiency as vector.  I checked this by looking at disassembled code.

#ifndef cvec_hpp
#define cvec_hpp
#include<vector>

using namespace std;    // so "vector" below is actually "std::vector", etc.

template<class T> class cvec : public vector<T>
{
public:     // typenames like iterator are also inherited

typedef typename cvec::size_type size_type;
typedef typename cvec::iterator iterator;
typedef typename cvec::difference_type difference_type;
typedef typename cvec::reference reference;
typedef typename cvec::const_reference const_reference;

cvec() {}

cvec(size_type n, const T& value = T()): vector<T>(n, value) {}

cvec(iterator i, iterator j): vector<T>(i, j) {}

reference operator[](difference_type i)
    {
    DASSERT(i >=0  && i < static_cast<difference_type>(this->size()));
    return vector<T>::operator[](i);
    }
const_reference operator[](difference_type i) const
    {
    DASSERT(i >=0 && i < static_cast<difference_type>(this->size()));
    return vector<T>::operator[](i);
    }
};
#endif  // cvec_hpppp
