// $main.cpp 1.5 milbo$ main routine for locating face features using stasm code
//-----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// A copy of the GNU General Public License is available at
// http://www.r-project.org/Licenses/
//-----------------------------------------------------------------------------

#include "all.hpp"

#define STASM_VERSION "version 1.5"

static const char *LOG_FILE_NAME = "log";

//-----------------------------------------------------------------------------
int
main (int argc, const char *argv[])
{
FILE *pLogFile = pOpenLogFile(LOG_FILE_NAME);   // lprintfs to screen and log file
#if _MSC_VER    // microsoft
SetMallocHandler();                             // assign func to be called if malloc fails
#endif
gsl_set_error_handler(GslErrorHandler);         // assign func to be called if GSL err

// get data directory, it will be needed to locate data files

char sBaseDir[SLEN], sDataDir[SLEN];
_splitpath(argv[0], NULL, sBaseDir, NULL, NULL);
if(sBaseDir == NULL || sBaseDir[0] == 0)
    strcpy(sDataDir, "../stasm/data");
else
    sprintf(sDataDir, "%s/../stasm/data", sBaseDir);

bool fViolaJones = false;

#if _MSC_VER // microsoft
    static const char *sUsage = "Usage: stasm [-v] ImageFile [ImageFile2 ...]\n"
    "\n"
    "-v  use the Viola Jones face detector (default is the Rowley detector)\n"
    "\n"
    "This is stasm " STASM_VERSION;
#else
    static const char *sUsage = "Usage: stasm ImageFile [ImageFile2 ...]\n"
    "\n"
    "This is stasm " STASM_VERSION;
#endif
while (--argc > 0 && (*++argv)[0] == '-')
    {
    switch (*(*argv + 1))
        {
#if _MSC_VER
        case 'v':
            fViolaJones = true;
            break;
#endif
        default:
            Err(sUsage);
        }
    }
if (argc < 1)
    Err(sUsage);

for (int iImage = 0; iImage < argc; iImage++)   // for each file on the cmd line
    {
    // read the image from disk

    const char *sImageFile = argv[iImage];
    lprintf("Processing %s ", sImageFile);
    RgbImage Img(sImageFile);

    // locate the face landmarks, put them into Shape

    SHAPE Shape = AsmSearch(Img, fViolaJones, sImageFile, sDataDir);

    // write Img, with Shape superimposed

    const int width = Img.width, height = Img.height;
    DrawShape(Img, Shape, C_DRED);
    CropImageToShape(Img, Shape);
    char sImage[SLEN], s[SLEN];
    _splitpath(sImageFile, NULL, NULL, sImage, NULL);
    sprintf(s, "results-%s-%s.bmp", (fViolaJones? "vj": "rowley"), sImage);
    WriteBmp(Img, s, VERBOSE);

    // Print coordinates of Shape to the log file.  But first convert
    // shape coords (with 0,0 at center of image) to standard image coords.

    Shape.col(VX) += width / 2;
    Shape.col(VY) = height / 2 - Shape.col(VY);
    Shape.print("\n", "%g ", LOG_FILE_NAME, pLogFile);
    }
Shutdown();
return 0;   // 0 means success
}
