// $misc.hpp 1.5 milbo$ miscellaneous definitions for stasm

#if !defined(misc_hpp)
#define misc_hpp

#ifndef byte
#define byte unsigned char
#endif

//-----------------------------------------------------------------------------
// DASSERT is only enabled if debugging, ASSERT is always enabled.
// Use DASSERT when the release code must be fast.

#if _DEBUG
#define DASSERT(x)  if (!(x)) Err("%s %d: FAIL %s", __FILE__, __LINE__, #x)
#else
#define DASSERT(x)
#endif
#define ASSERT(x)   if (!(x)) Err("%s %d: FAIL %s", __FILE__, __LINE__, #x)

//-----------------------------------------------------------------------------
#if _MSC_VER    // microsoft
    #if _DEBUG  // debugging enabled
    #include <crtdbg.h>
    #define malloc(size)      _malloc_dbg((size), _NORMAL_BLOCK, __FILE__, __LINE__)
    #define calloc(num, size) _calloc_dbg((num), (size), _NORMAL_BLOCK, __FILE__, __LINE__)
    #endif
#else           // gcc
    //TODO is there a better way than this of handling malloc fails under GCC?
    #define calloc(num, size) _calloc((num), (size))
    #define malloc(size)      _malloc(size)
#endif  // _MSC_VER

//-----------------------------------------------------------------------------
#define SHAPE Mat       // By convention this specifies a Nx2 matrix holding a shape.
                        // Coord 0,0 is the center of the image which makes it
                        // easier to deal with various image sizes (but 0,0 also
                        // means "unused landmark").

#define IS_ODD(i)   (((i) & 1)==1)
#define ABS(x)      (((x) < 0)? -(x): (x))
#define SQR(x)      ((x) * (x))
#define MAX(x,y)    (((x) > (y)) ? (x) : (y))
#define MIN(x,y)    (((x) < (y)) ? (x) : (y))
#define NELEMS(x)   int(sizeof(x) / sizeof((x)[0])) // nbr of elems in an array

#define ENTER_DEBUGGER  *(int *)99 = 99 // MS abort() doesn't always work, hence this macro

static const bool ENTER_DEBUGGER_ON_ERR = true;  // only applies if _DEBUG

static const bool QUIET   = false;      // values for fVerbose parameter
static const bool VERBOSE = true;

static const int SLEN = 256;    // generic string length
static const int FLEN = 64;     // shorter string len, used for file base names, etc.

//-----------------------------------------------------------------------------
// iRound() is "symmetrical" in that e.g. 1.5 becomes 1 and -1.5 becomes -1
// Inline routines have to be in the header file to be seen by microsoft linker.

static inline int iRound (double x)
{
if (x < 0)
    return int(x - 0.5);
else
    return int(x + 0.5);
}

// fEqual () returns true if x == y within reasonable tolerance
// If one or both of the numbers are NANs then the test fails (even if they
// are equal NANs).
//
// I'm just doing a straight comparism because GSL gsl_fcmp() tests ratios and
// is not useful for small numbers (because two small numbers can be close
// in absolute value but far in ratio).
//
// But I added the _isnan tests because the microsoft v6 compiler says NANs
// are equal to anything and also less than anything i.e NAN==x and NAN<x are true
// for any double x.  This is a compiler bug I believe (because any comparison to
// a NAN should give a NAN).

static inline bool fEqual (double x, double y, double MaxDiff=1e-10)
{
return !_isnan(x) && !_isnan(y) && ABS(x-y) < MaxDiff;
}

#endif // misc_hpp
