// $sparsemat.hpp 1.5 milbo$ routines for dealing with sparse matrices

#if !defined(sparsemat_hpp)
#define sparsemat_hpp

// Row layout is:
//      1st row: nrows, ncols, SPARSE_SYMMETRIC
//      N following rows: all diag entries
//      remaining rows: upper triangle entries
//
// Note that the lower triangular entries aren't stored (that's
// unneeded because the matrix is symmetric).
//
// In tSparseElem, we use shorts and floats rather than
// ints and doubles for a small speed improvement
// (TODO does this actually improve speed?)

typedef struct tSparseElem  // an element in a sparse array
    {
    short   iRow, iCol;
    float   Val;
    }
tSparseElem;

// special value for A(0,2) to mark an array as sparse symmetric

static const double SPARSE_SYMMETRIC = -9999;

// we use a define not a typedef here so we don't have to
// include the STL header files everywhere

#define SparseMat cvec<tSparseElem>

double Sparse_xAx(const Vec &x, const SparseMat &A);
void CopyMatToSparseMat(SparseMat &S, const Mat &M);
bool fSparseMat(const Mat &A);

#endif // sparsemat_hpp
